"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Verifier = void 0;
var logger_1 = require("../logger");
var promise_timeout_1 = require("promise-timeout");
var nativeVerifier_1 = require("./nativeVerifier");
var applyDefaults = function (options) { return (__assign({ timeout: 30000 }, options)); };
var Verifier = (function () {
    function Verifier(options) {
        this.options = applyDefaults(options);
    }
    Verifier.prototype.verify = function () {
        logger_1.default.info('Verifying Pact Files');
        logger_1.default.trace('verifier arguments', { options: this.options });
        return promise_timeout_1.timeout(nativeVerifier_1.verify(this.options), this.options.timeout).catch(function (err) {
            if (err instanceof promise_timeout_1.TimeoutError) {
                throw new Error("Timeout waiting for verification process to complete");
            }
            throw err;
        });
    };
    return Verifier;
}());
exports.Verifier = Verifier;
exports.default = (function (options) { return new Verifier(options); });
//# sourceMappingURL=verifier.js.map