"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadChecksums = void 0;
var chalk_1 = require("chalk");
var path = require("path");
var config_1 = require("./config");
var http_1 = require("./connectors/http");
var console_1 = require("./connectors/console");
var setup_1 = require("./setup");
function downloadChecksums() {
    console.log(chalk_1.default.gray("Downloading All Pact Standalone Binary Checksums."));
    return Promise.all(config_1.CONFIG.binaries.map(function (value) {
        return setup_1.setup(value.platform, value.arch)
            .then(function (allData) {
            return allData.filter(function (_a) {
                var checksumFilepath = _a.checksumFilepath;
                return checksumFilepath !== 'skip';
            });
        })
            .then(function (checksums) {
            return Promise.all(checksums.map(function (data) {
                return http_1.downloadFileRetry(data.checksumDownloadPath, data.checksumFilepath).then(function () {
                    console.log(chalk_1.default.green("Finished downloading checksum " + path.basename(data.checksumFilepath)));
                }, function (error) {
                    return console_1.throwError("Error downloading checksum from " + data.checksumDownloadPath + ": " + error);
                });
            }));
        });
    })).then(function () { return console.log(chalk_1.default.green('All checksums downloaded.')); }, function (e) { return console_1.throwError("Checksum Download Failed Unexpectedly: " + e); });
}
exports.downloadChecksums = downloadChecksums;
//# sourceMappingURL=downloadChecksums.js.map